/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.usm.api.contenttypes;

import java.util.*;

import com.openexchange.usm.api.exceptions.*;
import com.openexchange.usm.api.session.DataObject;
import com.openexchange.usm.api.session.Session;

/**
 * Provides all relevant information about the Contact content type.
 * 
 * @author ldo
 *
 */
public interface ContactContentType extends ContentType {

	/**
	 * Resolves mail recipients from the user's contacts. 
	 * @param toList
	 * @param session
	 * @param requestedFields
	 * @return
	 * @throws AuthenticationFailedException
	 * @throws OXCommunicationException
	 * @throws UnsupportedContentOperationException
	 */
	Map<String, DataObject[]> resolveRecipients(List<String> toList, Session session, BitSet requestedFields)
			throws AuthenticationFailedException, OXCommunicationException, UnsupportedContentOperationException, InternalUSMException;

	/**
	 * Gets the contacts picture as byte array. 
	 * @param dataObject
	 * @param pictureFormat
	 * @param maxSize
	 * @return
	 * @throws AuthenticationFailedException
	 * @throws OXCommunicationException
	 * @throws PictureConversionException
	 */
	byte[] getPictureData(DataObject dataObject, String pictureFormat, int maxSize)
			throws AuthenticationFailedException, OXCommunicationException, PictureConversionException;

	/**
	 * Gets all registered users from the user interface.
	 * @param session
	 * @param requestedFields
	 * @return
	 * @throws AuthenticationFailedException
	 * @throws OXCommunicationException
	 * @throws UnsupportedContentOperationException
	 * @throws InternalUSMException
	 */
	DataObject[] getAllUsers(Session session, BitSet requestedFields) throws AuthenticationFailedException,
			OXCommunicationException, UnsupportedContentOperationException, InternalUSMException;

	/**
	 * Gets all data for a specific user. 
	 * @param object
	 * @param requestedFields
	 * @return
	 * @throws AuthenticationFailedException
	 * @throws OXCommunicationException
	 * @throws UnsupportedContentOperationException
	 * @throws InternalUSMException
	 */
	void getUser(DataObject object, BitSet requestedFields) throws AuthenticationFailedException,
			OXCommunicationException, UnsupportedContentOperationException, InternalUSMException;
}
