/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.usm.database.ox;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import com.openexchange.exception.OXException;
import com.openexchange.groupware.update.Schema;
import com.openexchange.groupware.update.UpdateTask;
import com.openexchange.usm.util.Toolkit;

public abstract class USMBaseUpdateTask implements UpdateTask {
	/**
	 * The object type "TABLE"
	 */
	private static final String[] TYPES = { "TABLE" };

	/**
	 * Check a table's existence
	 * 
	 * @param tableName The table name to check
	 * @param dbmd The database's meta data
	 * @return <code>true</code> if table exists; otherwise <code>false</code>
	 * @throws SQLException If a SQL error occurs
	 */
	public static boolean tableExists(final String tableName, final DatabaseMetaData dbmd) throws SQLException {
		ResultSet resultSet = null;
		try {
			resultSet = dbmd.getTables(null, null, tableName, TYPES);
			return resultSet.next();
		} finally {
			Toolkit.close(resultSet);
		}
	}

	protected final USMUpdateTaskProvider _provider;

	public USMBaseUpdateTask(USMUpdateTaskProvider updateTaskProvider) {
		_provider = updateTaskProvider;
	}

	public int getPriority() {
		return UpdateTaskPriority.LOW.priority;
	}

	public void perform(Schema schema, int contextId) throws OXException {
		String taskName = getClass().getName();
		int i = taskName.lastIndexOf('.');
		if (i >= 0)
			taskName = taskName.substring(i + 1);
		_provider.getJournal().debug("Performing UpdateTask '" + taskName + "'(" + addedWithVersion() + ")");
		doPerform(schema, contextId);
		_provider.getJournal()
				.info("UpdateTask '" + taskName + "'(" + addedWithVersion() + ") successfully performed!");
	}

	protected abstract void doPerform(Schema schema, int contextId) throws OXException;

	protected void createTable(final String tablename, final String sqlCreate, final int contextId)
			throws OXException {
		Connection con = null;
		Statement stmt = null;
		try {
			con = _provider.getDatabaseService().getWritable(contextId);
			if (!tableExists(tablename, con.getMetaData())) {
				stmt = con.createStatement();
				stmt.executeUpdate(sqlCreate);
			}
		} catch (SQLException e) {
			throw USMUpdateExceptionCodes.SQL_ERROR.create(e);
		} finally {
			Toolkit.close(stmt);
			if (con != null)
				_provider.getDatabaseService().backWritable(contextId, con);
		}
	}
}
